@@ -6,7 +6,7 @@ from ufltestcase import UflTestCase, main
 
 from ufl import *
 
-from ufl.geometry import domain2dim
+from ufl.geometry import cellname2dim
 
 all_cells = (interval, triangle, tetrahedron, quadrilateral, hexahedron)
 
@@ -25,10 +25,12 @@ class ElementsTestCase(UflTestCase):
     def test_vector_galerkin(self):
         for cell in all_cells:
             dim = cell.d
+            #shape = () if dim == 1 else (dim,)
+            shape = (dim,)
             for p in range(1,10):
                 for family in ("Lagrange", "CG", "Discontinuous Lagrange", "DG"):
                     element = VectorElement(family, cell, p)
-                    self.assertEqual(element.value_shape(), (dim,))
+                    self.assertEqual(element.value_shape(), shape)
                     self.assertEqual(element, eval(repr(element)))
                     for i in range(dim):
                         c = element.extract_component(i)
@@ -37,10 +39,12 @@ class ElementsTestCase(UflTestCase):
     def test_tensor_galerkin(self):
         for cell in all_cells:
             dim = cell.d
+            #shape = () if dim == 1 else (dim,dim)
+            shape = (dim,dim)
             for p in range(1,10):
                 for family in ("Lagrange", "CG", "Discontinuous Lagrange", "DG"):
                     element = TensorElement(family, cell, p)
-                    self.assertEqual(element.value_shape(), (dim,dim))
+                    self.assertEqual(element.value_shape(), shape)
                     self.assertEqual(element, eval(repr(element)))
                     for i in range(dim):
                         for j in range(dim):
@@ -52,6 +56,10 @@ class ElementsTestCase(UflTestCase):
             dim = cell.d
             for p in range(1,10):
                 for s in (None, True, {(0,1): (1,0)}):
+                    # Symmetry dict is invalid for interval cell
+                    if isinstance(s,dict) and cell == interval:
+                        continue
+
                     for family in ("Lagrange", "CG", "Discontinuous Lagrange", "DG"):
                         if isinstance(s, dict):
                             element = TensorElement(family, cell, p, shape=(dim,dim), symmetry=s)
@@ -116,6 +124,19 @@ class ElementsTestCase(UflTestCase):
             self.assertEqual(TH1, eval(repr(TH2)))
             self.assertEqual(TH2, eval(repr(TH1)))
 
+    def test_nested_mixed(self):
+        for cell in (triangle, tetrahedron):
+            dim = cell.d
+            velement = VectorElement("CG", cell, 2)
+            pelement = FiniteElement("CG", cell, 1)
+            TH1 = MixedElement((velement, pelement), pelement)
+            TH2 = velement * pelement * pelement
+            self.assertEqual(TH1.value_shape(), (dim+2,))
+            self.assertEqual(TH2.value_shape(), (dim+2,))
+            self.assertEqual(repr(TH1), repr(TH2))
+            self.assertEqual(TH1, eval(repr(TH2)))
+            self.assertEqual(TH2, eval(repr(TH1)))
+
     def test_quadrature_scheme(self):
         for cell in (triangle, tetrahedron):
             for q in (None, 1, 2, 3):
@@ -123,22 +144,26 @@ class ElementsTestCase(UflTestCase):
                 self.assertEqual(element.quadrature_scheme(), q)
                 self.assertEqual(element, eval(repr(element)))
 
-    def test_invalid_cell(self):
-        from ufl.geometry import as_cell
-        for cell in (triangle, as_cell(None)):
-            element = FiniteElement("CG", cell, 1)
+    def test_missing_cell(self):
+        # These special cases are here to allow missing
+        # cell in PyDOLFIN Constant and Expression
+        for cell in (triangle, None):
+            element = FiniteElement("Real", cell, 0)
+            self.assertEqual(element, eval(repr(element)))
+            element = FiniteElement("Undefined", cell, None)
+            self.assertEqual(element, eval(repr(element)))
+            element = VectorElement("Lagrange", cell, 1, dim=2)
+            self.assertEqual(element, eval(repr(element)))
+            element = TensorElement("DG", cell, 1, shape=(2,2))
             self.assertEqual(element, eval(repr(element)))
-            #element = VectorElement("CG", cell, 1) # invalid
 
     def test_invalid_degree(self):
-        from ufl.geometry import as_cell
         cell = triangle
         for degree in (1, None):
             element = FiniteElement("CG", cell, degree)
             self.assertEqual(element, eval(repr(element)))
             element = VectorElement("CG", cell, degree)
             self.assertEqual(element, eval(repr(element)))
-            print repr(element)
 
 if __name__ == "__main__":
     main()
