@@ -34,10 +34,9 @@ The development version can be found in the repository at
 
 A very brief overview of the language contents follows:
 
-* Euclidean spaces::
+* Domains::
 
-    Space,
-    R1, R2, R3,
+    Domain, Region
 
 * Cells::
 
@@ -50,7 +49,8 @@ A very brief overview of the language contents follows:
 
     FiniteElement,
     MixedElement, VectorElement, TensorElement
-    EnrichedElement, RestrictedElement
+    EnrichedElement, RestrictedElement,
+    TensorProductElement
 
 * Arguments::
 
@@ -70,8 +70,11 @@ A very brief overview of the language contents follows:
 
 * Geometric quantities::
 
-    SpatialCoordinate, FacetNormal, Circumradius,
-    CellVolume, CellSurfaceArea, FacetArea
+    SpatialCoordinate, FacetNormal,
+    CellVolume, Circumradius, CellSurfaceArea,
+    FacetArea, FacetDiameter,
+    LocalCoordinate, GeometryJacobi,
+    GeometryJacobiDeterminant, InverseGeometryJacobi
 
 * Indices::
 
@@ -89,7 +92,7 @@ A very brief overview of the language contents follows:
 
 * Tensor algebra operators::
 
-    outer, inner, dot, cross,
+    outer, inner, dot, cross, perp,
     det, inv, cofac,
     transpose, tr, diag, diag_vector,
     dev, skew, sym
@@ -106,8 +109,12 @@ A very brief overview of the language contents follows:
 
 * Nonlinear functions::
 
-    abs, sign, sqrt, exp, ln, erf,
-    cos, sin, tan, acos, asin, atan,
+    Max, Min,
+    abs, sign, sqrt,
+    exp, ln, erf,
+    cos, sin, tan,
+    acos, asin, atan,
+    cosh, sinh, tanh,
     bessel_J, bessel_Y, bessel_I, bessel_K
 
 * Discontinuous Galerkin operators:
@@ -115,11 +122,14 @@ A very brief overview of the language contents follows:
 
 * Conditional operators::
 
-    eq, ne, le, ge, lt, gt, And, Or, Not, conditional
+    eq, ne, le, ge, lt, gt,
+    <, >, <=, >=,
+    And, Or, Not,
+    conditional
 
 * Integral measures::
 
-    dx, ds, dS, dE, dc
+    dx, ds, dS, dP, dE, dc
 
 * Form transformations::
 
@@ -128,7 +138,7 @@ A very brief overview of the language contents follows:
     sensitivity_rhs, derivative
 """
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes and Anders Logg
+# Copyright (C) 2008-2013 Martin Sandve Alnes and Anders Logg
 #
 # This file is part of UFL.
 #
@@ -148,9 +158,9 @@ A very brief overview of the language contents follows:
 # Modified by Kristian B. Oelgaard, 2009, 2011
 # Modified by Anders Logg, 2009.
 #
-# Last changed: 2011-12-07
+# Last changed: 2013-03-15
 
-__version__ = "1.0.0"
+__version__ = "1.2.0"
 
 ########## README
 # Imports here should be what the user sees when doing "from ufl import *",
@@ -168,15 +178,23 @@ from ufl.log import get_handler, get_logger, set_handler, set_level, add_logfile
     UFLException, DEBUG, INFO, WARNING, ERROR, CRITICAL
 
 # Types for geometric quantities
-from ufl.geometry import Cell, Space, SpatialCoordinate, FacetNormal, Circumradius, \
-                         CellVolume, CellSurfaceArea, FacetArea
+from ufl.geometry import (Cell, ProductCell,
+     SpatialCoordinate, FacetNormal,
+     CellVolume, Circumradius, CellSurfaceArea,
+     FacetArea, FacetDiameter,
+     LocalCoordinate, GeometryJacobi,
+     GeometryJacobiDeterminant, InverseGeometryJacobi)
+
+# Types for domain description
+from ufl.domains import Domain, Region
 
 # Finite elements classes
 from ufl.finiteelement import FiniteElementBase, FiniteElement, \
-    MixedElement, VectorElement, TensorElement, EnrichedElement, RestrictedElement
+    MixedElement, VectorElement, TensorElement, EnrichedElement, \
+    RestrictedElement, TensorProductElement
 
 # Hook to extend predefined element families
-from ufl.elementlist import register_element, show_elements #, ufl_elements
+from ufl.finiteelement.elementlist import register_element, show_elements #, ufl_elements
 
 # Arguments
 from ufl.argument import Argument, TestFunction, TrialFunction, \
@@ -205,28 +223,27 @@ from ufl.tensors import unit_vector, unit_vectors, unit_matrix, unit_matrices
 
 # Operators
 from ufl.operators import rank, shape, \
-                       outer, inner, dot, cross, \
+                       outer, inner, dot, cross, perp, \
                        det, inv, cofac, \
                        transpose, tr, diag, diag_vector, \
                        dev, skew, sym, \
                        sqrt, exp, ln, erf, \
-                       cos, sin, tan, acos, asin, atan, \
+                       cos, sin, tan, \
+                       acos, asin, atan, \
+                       cosh, sinh, tanh, \
                        bessel_J, bessel_Y, bessel_I, bessel_K, \
                        eq, ne, le, ge, lt, gt, And, Or, Not, \
-                       conditional, sign, \
+                       conditional, sign, Max, Min, \
                        variable, diff, \
                        Dx,  grad, div, curl, rot, nabla_grad, nabla_div, Dn, exterior_derivative, \
                        jump, avg, \
                        elem_mult, elem_div, elem_pow, elem_op
 
-# Lifting
-from ufl.lifting import LiftingFunction, LiftingOperator
-
 # Form class
 from ufl.form import Form
 
 # Integral classes
-from ufl.integral import Integral, Measure, register_domain_type
+from ufl.integral import Integral, Measure, register_domain_type, ProductMeasure
 
 # Representations of transformed forms
 from ufl.formoperators import replace, derivative, action, energy_norm, rhs, lhs,\
@@ -237,8 +254,7 @@ from ufl.objects import \
     vertex, interval, triangle, tetrahedron, \
     quadrilateral, hexahedron, facet, cell1D, cell2D, cell3D, \
     i, j, k, l, p, q, r, s, \
-    R1, R2, R3, \
-    dx, ds, dS, dE, dc
+    dx, ds, dS, dP, dE, dc
 
 # Useful constants
 from math import e, pi
@@ -247,10 +263,16 @@ __all__ = [
     'product',
     'get_handler', 'get_logger', 'set_handler', 'set_level', 'add_logfile',
     'UFLException', 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL',
-    'Cell', 'Space', 'SpatialCoordinate', 'FacetNormal', 'CellVolume',
-    'Circumradius', 'CellSurfaceArea', 'FacetArea',
+    'Cell', 'ProductCell',
+    'SpatialCoordinate', 'FacetNormal',
+    'CellVolume', 'Circumradius', 'CellSurfaceArea',
+    'FacetArea', 'FacetDiameter',
+    'LocalCoordinate', 'GeometryJacobi',
+    'GeometryJacobiDeterminant', 'InverseGeometryJacobi',
+    'Domain', 'Region',
     'FiniteElementBase', 'FiniteElement',
-    'MixedElement', 'VectorElement', 'TensorElement', 'EnrichedElement', 'RestrictedElement',
+    'MixedElement', 'VectorElement', 'TensorElement', 'EnrichedElement',
+    'RestrictedElement', 'TensorProductElement',
     'register_element', 'show_elements',
     'Argument', 'TestFunction', 'TrialFunction',
     'Arguments', 'TestFunctions', 'TrialFunctions',
@@ -262,25 +284,25 @@ __all__ = [
     'as_tensor', 'as_vector', 'as_matrix', 'relabel',
     'unit_vector', 'unit_vectors', 'unit_matrix', 'unit_matrices',
     'rank', 'shape',
-    'outer', 'inner', 'dot', 'cross',
+    'outer', 'inner', 'dot', 'cross', 'perp',
     'det', 'inv', 'cofac',
     'transpose', 'tr', 'diag', 'diag_vector', 'dev', 'skew', 'sym',
     'sqrt', 'exp', 'ln', 'erf',
-    'cos', 'sin', 'tan', 'acos', 'asin', 'atan',
+    'cos', 'sin', 'tan',
+    'acos', 'asin', 'atan',
+    'cosh', 'sinh', 'tanh',
     'bessel_J', 'bessel_Y', 'bessel_I', 'bessel_K',
     'eq', 'ne', 'le', 'ge', 'lt', 'gt', 'And', 'Or', 'Not',
-    'conditional', 'sign',
+    'conditional', 'sign', 'Max', 'Min',
     'variable', 'diff',
     'Dx', 'grad', 'div', 'curl', 'rot', 'nabla_grad', 'nabla_div', 'Dn', 'exterior_derivative',
     'jump', 'avg',
     'elem_mult', 'elem_div', 'elem_pow', 'elem_op',
-    'LiftingFunction', 'LiftingOperator',
     'Form',
-    'Integral', 'Measure', 'register_domain_type',
+    'Integral', 'Measure', 'register_domain_type', 'ProductMeasure',
     'replace', 'derivative', 'action', 'energy_norm', 'rhs', 'lhs',
     'system', 'functional', 'adjoint', 'sensitivity_rhs',
-    'R1', 'R2', 'R3',
-    'dx', 'ds', 'dS', 'dE', 'dc',
+    'dx', 'ds', 'dS', 'dP', 'dE', 'dc',
     'vertex', 'interval', 'triangle', 'tetrahedron',
     'quadrilateral', 'hexahedron', 'facet',
     'cell1D', 'cell2D', 'cell3D',
