@@ -1,6 +1,6 @@
 "This module collects algorithms and utility functions operating on UFL objects."
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -49,14 +49,24 @@ from ufl.algorithms.predicates import is_multilinear
 from ufl.algorithms.checks import validate_form
 
 # Utilites for modifying expressions and forms
-from ufl.algorithms.transformations import transform, MultiFunction, Transformer, ReuseTransformer, apply_transformer, \
-                                           ufl2ufl, ufl2uflcopy, \
-                                           replace, flatten, strip_variables, \
-                                           expand_compounds, \
-                                           mark_duplications, \
-                                           estimate_max_polynomial_degree, estimate_total_polynomial_degree
+from ufl.algorithms.multifunction import MultiFunction
+from ufl.algorithms.transformer import Transformer, is_post_handler, \
+                                       transform, transform_integrands, apply_transformer, \
+                                       ReuseTransformer, ufl2ufl, \
+                                       CopyTransformer, ufl2uflcopy, \
+                                       VariableStripper, strip_variables
+from ufl.algorithms.replace import Replacer, replace
+from ufl.algorithms.expand_compounds import CompoundExpander, expand_compounds, \
+                                            CompoundExpanderPreDiff, expand_compounds_prediff, \
+                                            CompoundExpanderPostDiff, expand_compounds_postdiff
+from ufl.algorithms.estimate_degrees import MaxDegreeEstimator, estimate_max_polynomial_degree, \
+                                            SumDegreeEstimator, estimate_total_polynomial_degree
+from ufl.algorithms.argument_dependencies import ArgumentDependencyExtracter, extract_argument_dependencies, NotMultiLinearException
+from ufl.algorithms.deprecated import TreeFlattener, flatten, \
+                                      DuplicationMarker, mark_duplications, \
+                                      DuplicationPurger, purge_duplications
 from ufl.algorithms.renumbering import renumber_indices
-from ufl.algorithms.expand_indices import expand_indices, expand_indices2, purge_list_tensors
+from ufl.algorithms.expand_indices import expand_indices, purge_list_tensors
 from ufl.algorithms.propagate_restrictions import propagate_restrictions
 
 # Utilities for transforming complete Forms into other Forms
@@ -71,7 +81,7 @@ from ufl.algorithms.ad import expand_derivatives #, compute_diff, propagate_spat
 from ufl.algorithms.graph import Graph, format_graph, rebuild_tree, partition # TODO: Add more imports here
 
 # Utilities for tuple notation
-from ufl.algorithms.tuplenotation import tuple2form, as_form
+from ufl.algorithms.tuplenotation import as_form
 
 # Utilities for UFL object printing
 from ufl.algorithms.printing import integral_info, form_info, tree_format
