@@ -1,6 +1,6 @@
 """Utility algorithms for inspection of and information extraction from UFL objects in various ways."""
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -21,13 +21,13 @@
 # Modified by Johan Hake, 2010.
 #
 # First added:  2008-03-14
-# Last changed: 2011-10-14
+# Last changed: 2013-01-02
 
 from itertools import izip, chain
+from collections import namedtuple
 
 from ufl.log import error, warning, info
 from ufl.assertions import ufl_assert
-from ufl.common import lstr, dstr, UFLTypeDefaultDict
 from ufl.sorting import topological_sorting
 
 from ufl.expr import Expr
@@ -36,11 +36,10 @@ from ufl.finiteelement import MixedElement, RestrictedElement
 from ufl.argument import Argument
 from ufl.coefficient import Coefficient
 from ufl.variable import Variable
-from ufl.tensors import ListTensor, ComponentTensor
-from ufl.indexing import Indexed, Index, MultiIndex
+from ufl.indexing import Index, MultiIndex
+from ufl.domains import Region, Domain
+from ufl.integral import Measure, Integral
 from ufl.form import Form
-from ufl.integral import Integral, Measure
-from ufl.classes import terminal_classes, nonterminal_classes
 from ufl.algorithms.traversal import iter_expressions, post_traversal, post_walk, traverse_terminals
 
 # Domain types (should probably be listed somewhere else)
@@ -84,18 +83,18 @@ def extract_terminals(a):
                  for o in post_traversal(e) \
                  if isinstance(o, Terminal))
 
-def cmp_counted(x, y):
-    return cmp(x._count, y._count)
+def sorted_by_count(seq):
+    return sorted(seq, key=lambda x: x._count)
 
 def extract_arguments(a):
     """Build a sorted list of all arguments in a,
     which can be a Form, Integral or Expr."""
-    return sorted(extract_type(a, Argument), cmp=cmp_counted)
+    return sorted_by_count(extract_type(a, Argument))
 
 def extract_coefficients(a):
     """Build a sorted list of all coefficients in a,
     which can be a Form, Integral or Expr."""
-    return sorted(extract_type(a, Coefficient), cmp=cmp_counted)
+    return sorted_by_count(extract_type(a, Coefficient))
 
 def extract_arguments_and_coefficients(a):
     """Build two sorted lists of all arguments and coefficients
@@ -127,8 +126,8 @@ The arguments found are:\n%s""" % "\n".join("  %s" % f for f in coefficients)
         error(msg)
 
     # Passed checks, so we can safely sort the instances by count
-    arguments = sorted(arguments, cmp=cmp_counted)
-    coefficients = sorted(coefficients, cmp=cmp_counted)
+    arguments = sorted_by_count(arguments)
+    coefficients = sorted_by_count(coefficients)
 
     return arguments, coefficients
 
@@ -160,7 +159,7 @@ def _extract_coefficients(a):
             s.add(o)
     post_walk(a, func)
     # sort by count
-    return sorted(s, cmp=cmp_counted)
+    return sorted_by_count(s)
 
 def extract_elements(form):
     "Build sorted tuple of all elements used in form."
@@ -173,9 +172,7 @@ def extract_unique_elements(form):
 
 def extract_sub_elements(elements):
     "Build sorted tuple of all sub elements (including parent element)."
-    # FIXME: Why do we have the isinstance here?
-    sub_elements = tuple(chain(*[e.sub_elements() for e in elements\
-                                     if isinstance(e, (MixedElement, RestrictedElement))]))
+    sub_elements = tuple(chain(*[e.sub_elements() for e in elements]))
     if not sub_elements: return tuple(elements)
     return tuple(elements) + extract_sub_elements(sub_elements)
 
@@ -278,54 +275,71 @@ def extract_domain_data(form):
     "Extract the domain_data attached to integrals of each domain type in form."
     domain_data = {}
     for integral in form.integrals():
-        domain_type = integral.measure().domain_type()
-        data = integral.measure().domain_data()
+        domain_type = integral.domain_type()
+        data = integral.domain_data()
         # Check that there is only one domain_data object for each integral type
         existing_data = domain_data.get(domain_type)
         if existing_data is None:
+            # Got no data before, store this one. May be None, that's fine.
             domain_data[domain_type] = data
-        else:
-            # Not assuming anything about domain_data type, not even an equals operator!
-            ufl_assert(existing_data is data,
-                       "Found two domain data objects for same domain type.")
+        elif data is None:
+            # Already got data, getting no data is ok but don't overwrite.
+            pass
+        elif existing_data is not data:
+            # NB! Using 'is' because we're not assuming anything about domain_data type, not even an equals operator!
+            error("Found two domain data objects for same domain type '%s', only one is allowed." % str(domain_type))
     return domain_data
 
 def extract_num_sub_domains(form):
     "Extract the upper limit of sub domain ids for each domain type."
     num_domains = {}
     for integral in form.integrals():
-        domain_type = integral.measure().domain_type()
-        domain_id = integral.measure().domain_id()
-        num_domains[domain_type] = max(num_domains.get(domain_type, 0), domain_id + 1)
+        domain_type = integral.domain_type()
+        domain_id = integral.domain_id()
+
+        # TODO: This may need some redesign
+        max_domain_id = None
+        if isinstance(domain_id, int):
+            max_domain_id = domain_id
+        elif isinstance(domain_id, Region):
+            max_domain_id = max(domain_id.subdomain_ids())
+        elif isinstance(domain_id, Domain):
+            max_domain_id = None
+
+        if max_domain_id is not None:
+            num_domains[domain_type] = max(num_domains.get(domain_type, 0), max_domain_id + 1)
     return num_domains
 
-def extract_integral_data(form):
-    """
-    Extract integrals from form stored by integral type and sub
-    domain, stored as a list of tuples
-
+class IntegralData(object):
+    """Utility class with the members
         (domain_type, domain_id, integrals, metadata)
 
     where metadata is an empty dictionary that may be used for
-    associating metadata with each tuple.
+    associating metadata with each object.
     """
+    __slots__ = ('domain_type', 'domain_id', 'integrals', 'metadata')
+    def __init__(self, domain_type, domain_id, integrals, metadata):
+        self.domain_type = domain_type
+        self.domain_id = domain_id
+        self.integrals = integrals
+        self.metadata = metadata
 
-    # Extract integral data
-    integral_data = {}
-    for integral in form.integrals():
-        domain_type = integral.measure().domain_type()
-        domain_id = integral.measure().domain_id()
-        if (domain_type, domain_id) in integral_data:
-            integral_data[(domain_type, domain_id)].append(integral)
-        else:
-            integral_data[(domain_type, domain_id)] = [integral]
+    def __lt__(self, other):
+        # To preserve behaviour of extract_integral_data:
+        return ((self.domain_type, self.domain_id, self.integrals, self.metadata)
+                < (other.domain_type, other.domain_id, other.integrals, other.metadata))
 
-    # Note that this sorting thing here is pretty interesting. The
-    # domain types happen to be in alphabetical order (cell, exterior,
-    # interior, and macro) so we can just sort... :-)
+    def __eq__(self, other):
+        # Currently only used for tests:
+        return (self.domain_type == other.domain_type and
+                self.domain_id == other.domain_id and
+                self.integrals == other.integrals and
+                self.metadata == other.metadata)
 
-    # Sort by domain type and number
-    return sorted((d, n, i, {}) for ((d, n), i) in integral_data.iteritems())
+    def __str__(self):
+        return "IntegralData object over domain (%s, %s), with integrals:\n%s\nand metadata:\n%s" % (
+            self.domain_type, self.domain_id,
+            '\n\n'.join(map(str,self.integrals)), self.metadata)
 
 def sort_elements(elements):
     """
