@@ -2,7 +2,7 @@
 
 from __future__ import with_statement
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -23,22 +23,16 @@ from __future__ import with_statement
 # Modified by Marie E. Rognes, 2011.
 #
 # First added:  2008-03-14
-# Last changed: 2011-07-01
+# Last changed: 2012-12-11
 
-import os
-import time
-import re
-
-from ufl.log import error, warning, info
+import os, re
+from ufl.log import error, warning
 from ufl.assertions import ufl_assert
 from ufl.form import Form
 from ufl.finiteelement import FiniteElementBase
 from ufl.expr import Expr
 from ufl.argument import Argument
 from ufl.coefficient import Coefficient
-from ufl.algorithms.formdata import FormData
-from ufl.algorithms.checks import validate_form
-from ufl.algorithms.tuplenotation import as_form
 
 class FileData(object):
     def __init__(self):
@@ -59,60 +53,55 @@ To help you find the location of the error, a temporary script
 '%s'
 has been created and will now be executed with debug output enabled:"""
 
+def replace_include_statements(code):
+    "Replace '#include foo.ufl' statements with contents of foo.ufl."
+    if "#include" in code:
+        lines = code.split("\n")
+        newlines = []
+        regexp = re.compile(r"^#include (.*)$")
+        for l in lines:
+            m = regexp.search(l)
+            if m:
+                fn = m.groups()[0]
+                newlines.append("# --- begin %s" % fn)
+                newlines.extend(open(fn).readlines())
+                newlines.append("# --- end %s" % fn)
+            else:
+                newlines.append(l)
+        return "\n".join(l.rstrip() for l in newlines)
+    return code
+
 def read_ufl_file(filename):
-    "Load a .ufl file with elements, coefficients and forms."
+    "Read a .ufl file, handling file extension, file existance, and #include replacement."
     if not os.path.exists(filename) and filename[-4:] != ".ufl":
         filename = filename + ".ufl"
     if not os.path.exists(filename):
         error("File '%s' doesn't exist." % filename)
-
-    # Read form file and prepend import
     with open(filename) as f:
-        fcode = f.read()
-        # Replace #include foo.ufl statements with contents of foo.ufl
-        if "#include" in fcode:
-            lines = fcode.split("\n")
-            newlines = []
-            regexp = re.compile(r"^#include (.*)$")
-            for l in lines:
-                m = regexp.search(l)
-                if m:
-                    fn = m.groups()[0]
-                    newlines.append("# --- begin %s" % fn)
-                    newlines.extend(open(fn).readlines())
-                    newlines.append("# --- end %s" % fn)
-                else:
-                    newlines.append(l)
-            fcode = "\n".join(l.rstrip() for l in newlines)
-
-    return fcode
-
-def execute_ufl_code(filename):
-    # Read code from file
-    fcode = read_ufl_file(filename)
+        code = replace_include_statements(f.read())
+    return code
 
+def execute_ufl_code(uflcode, filename):
     # Execute code
     namespace = {}
     try:
-        code = "from ufl import *\n" + fcode
-        exec code in namespace
+        pycode = "from ufl import *\n" + uflcode
+        exec pycode in namespace
     except:
-        # Dump code for debugging if this fails
+        # Dump python code for debugging if this fails
         basename = os.path.splitext(os.path.basename(filename))[0]
         basename = "%s_debug" % basename
         pyname = "%s.py" % basename
-        code = "#!/usr/bin/env python\nfrom ufl import *\nset_level(DEBUG)\n" + fcode
+        pycode = "#!/usr/bin/env python\nfrom ufl import *\nset_level(DEBUG)\n" + uflcode
         with file(pyname, "w") as f:
-            f.write(code)
+            f.write(pycode)
         warning(infostring % pyname)
         m = __import__(basename)
         error("An error occured, aborting load_forms.")
     return namespace
 
-def load_ufl_file(filename):
-    "Load a .ufl file with elements, coefficients and forms."
-    namespace = execute_ufl_code(filename)
-
+def interpret_ufl_namespace(namespace):
+    "Takes a namespace dict from an executed ufl file and converts it to a FileData object."
     # Object to hold all returned data
     ufd = FileData()
 
@@ -121,12 +110,6 @@ def load_ufl_file(filename):
     # because we need to distinguish between instances,
     # and not just between objects with different values.
     for name, value in namespace.iteritems():
-        # Map tuples to forms if possible
-        if isinstance(value, tuple):
-            value = as_form(value)
-            if not isinstance(value, Form):
-                continue
-
         # Store objects by reserved name OR instance id
         reserved_names = ("unknown",) # Currently only one reserved name
         if name in reserved_names:
@@ -144,17 +127,20 @@ def load_ufl_file(filename):
     forms = namespace.get("forms")
     if forms is None:
         # Get forms from object_by_name, which has already mapped tuple->Form where needed
-        a_form = ufd.object_by_name.get("a")
-        L_form = ufd.object_by_name.get("L")
-        M_form = ufd.object_by_name.get("M")
+        def get_form(name):
+            form = ufd.object_by_name.get(name)
+            return form if isinstance(form, Form) else None
+        a_form = get_form("a")
+        L_form = get_form("L")
+        M_form = get_form("M")
         forms = [a_form, L_form, M_form]
         # Add forms F and J if not "a" and "L" are used
         if a_form is None or L_form is None:
-            F_form = ufd.object_by_name.get("F")
-            J_form = ufd.object_by_name.get("J")
+            F_form = get_form("F")
+            J_form = get_form("J")
             forms += [F_form, J_form]
         # Remove Nones
-        forms = [f for f in forms if f is not None]
+        forms = [f for f in forms if isinstance(f, Form)]
     ufd.forms = forms
 
     # Validate types
@@ -202,6 +188,13 @@ def load_ufl_file(filename):
     # Return file data
     return ufd
 
+def load_ufl_file(filename):
+    "Load a .ufl file with elements, coefficients and forms."
+    # Read code from file and execute it
+    uflcode = read_ufl_file(filename)
+    namespace = execute_ufl_code(uflcode, filename)
+    return interpret_ufl_namespace(namespace)
+
 def load_forms(filename):
     "Return a list of all forms in a file."
     ufd = load_ufl_file(filename)
