@@ -1,6 +1,6 @@
 """Forward mode AD implementation."""
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -20,10 +20,12 @@
 # Modified by Anders Logg, 2009.
 # Modified by Garth N. Wells, 2010.
 # Modified by Kristian B. Oelgaard, 2011
+# Modified by Jan Blechta, 2012.
 #
 # First added:  2008-08-19
-# Last changed: 2011-10-24
+# Last changed: 2012-10-03
 
+from itertools import izip
 from math import pi
 from ufl.log import error, warning, debug
 from ufl.assertions import ufl_assert
@@ -31,32 +33,30 @@ from ufl.common import unzip, subdict, lstr
 from ufl.indexutils import unique_indices
 
 # All classes:
-from ufl.terminal import Terminal, Tuple
+from ufl.terminal import Terminal
+from ufl.operatorbase import Tuple
 from ufl.constantvalue import ConstantValue, Zero, IntValue, Identity,\
     is_true_ufl_scalar, is_ufl_scalar
 from ufl.variable import Variable
-from ufl.coefficient import ConstantBase
-from ufl.indexing import MultiIndex, Indexed, Index, indices
+from ufl.coefficient import ConstantBase, Coefficient, FormArgument
+from ufl.indexing import MultiIndex, Index, FixedIndex, indices
+from ufl.indexed import Indexed
 from ufl.indexsum import IndexSum
-from ufl.tensors import ListTensor, ComponentTensor, as_tensor, as_scalar
+from ufl.tensors import ListTensor, ComponentTensor, as_tensor, as_scalar, unit_indexed_tensor, unwrap_list_tensor
 from ufl.algebra import Sum, Product, Division, Power, Abs
 from ufl.tensoralgebra import Transposed, Outer, Inner, Dot, Cross, Trace, \
     Determinant, Inverse, Deviatoric, Cofactor
 from ufl.mathfunctions import MathFunction, Sqrt, Exp, Ln, Cos, Sin, Tan, Acos, Asin, Atan, Erf, BesselJ, BesselY, BesselI, BesselK
 from ufl.restriction import Restricted, PositiveRestricted, NegativeRestricted
 from ufl.differentiation import Derivative, CoefficientDerivative,\
-    SpatialDerivative, VariableDerivative
+    VariableDerivative, Grad
 from ufl.conditional import EQ, NE, LE, GE, LT, GT, Conditional
 
-# Lists of all Expr classes
-from ufl.classes import terminal_classes
-
-from ufl.operators import dot, inner, outer, lt, eq, conditional, sign
-from ufl.operators import sqrt, exp, ln, cos, sin, tan, acos, asin, atan, \
+from ufl.operators import dot, inner, outer, lt, eq, conditional, sign, \
+    sqrt, exp, ln, cos, sin, tan, cosh, sinh, tanh, acos, asin, atan, \
     erf, bessel_J, bessel_Y, bessel_I, bessel_K
-from ufl.algorithms.traversal import iter_expressions
-from ufl.algorithms.analysis import extract_type
-from ufl.algorithms.transformations import expand_compounds, Transformer, transform, transform_integrands
+from ufl.algorithms.transformer import Transformer
+
 
 class ForwardAD(Transformer):
     def __init__(self, spatial_dim, var_shape, var_free_indices, var_index_dimensions, cache=None):
@@ -156,7 +156,6 @@ class ForwardAD(Transformer):
         # (kind of cumbersome this... any simpler way?)
 
         sh = o.shape()
-
         fi = o.free_indices()
         idims = dict(o.index_dimensions())
 
@@ -167,28 +166,34 @@ class ForwardAD(Transformer):
                 fi = unique_indices(fi + (i,))
                 idims[i] = self._var_index_dimensions[i]
 
+        # Create a 1 with index annotations
         one = IntValue(1, (), fi, idims)
-        if not sh:
-            return one
 
         res = None
-        ind1 = ()
-        ind2 = ()
-        for d in sh:
-            i, j = indices(2)
-            dij = Identity(d)[i, j]
-            if res is None:
-                res = dij
-            else:
-                res *= dij
-            ind1 += (i,)
-            ind2 += (j,)
+        if sh == ():
+            return one
+        elif len(sh) == 1:
+            # FIXME: If sh == (1,), I think this will get the wrong shape?
+            # I think we should make sure sh=(1,...,1) is always converted to () early.
+            fp = Identity(sh[0])
+        else:
+            ind1 = ()
+            ind2 = ()
+            for d in sh:
+                i, j = indices(2)
+                dij = Identity(d)[i, j]
+                if res is None:
+                    res = dij
+                else:
+                    res *= dij
+                ind1 += (i,)
+                ind2 += (j,)
+            fp = as_tensor(res, ind1 + ind2)
 
-        allind = ind1 + ind2
-
-        fp = as_tensor(res, allind)
+        # Apply index annotations
         if fi:
             fp *= one
+
         return fp
 
     # --- Default rules
@@ -311,7 +316,7 @@ class ForwardAD(Transformer):
         # Get operands and their derivatives
         ops2, dops2 = unzip(ops)
         o = self.reuse_if_possible(o, *ops2)
-        for (i, op) in enumerate(ops):
+        for i in xrange(len(ops)):
             # Get scalar representation of differentiated value of operand i
             dop = dops2[i]
             dop, ii = as_scalar(dop)
@@ -440,6 +445,26 @@ class ForwardAD(Transformer):
         op = fp*2.0/(cos(2.0*f) + 1.0)
         return (o, op)
 
+    def cosh(self, o, a):
+        f, fp = a
+        o = self.reuse_if_possible(o, f)
+        op = fp*sinh(f)
+        return (o, op)
+
+    def sinh(self, o, a):
+        f, fp = a
+        o = self.reuse_if_possible(o, f)
+        op = fp*cosh(f)
+        return (o, op)
+
+    def tanh(self, o, a):
+        f, fp = a
+        o = self.reuse_if_possible(o, f)
+        def sech(y):
+            return (2.0*cosh(y)) / (cosh(2.0*y) + 1.0)
+        op = fp*sech(f)**2
+        return (o, op)
+
     def acos(self, o, a):
         f, fp = a
         o = self.reuse_if_possible(o, f)
@@ -527,30 +552,33 @@ class ForwardAD(Transformer):
 
     def binary_condition(self, o, l, r):
         o = self.reuse_if_possible(o, l[0], r[0])
-        if any(not isinstance(op[1], Zero) for op in (l, r)):
-            warning("Differentiating a conditional with a condition "\
-                        "that depends on the differentiation variable."\
-                        "This is probably not a good idea!")
+        #if any(not (isinstance(op[1], Zero) or op[1] is None) for op in (l, r)):
+        #    warning("Differentiating a conditional with a condition "\
+        #                "that depends on the differentiation variable."\
+        #                "Assuming continuity of conditional. The condition "\
+        #                "will not be differentiated.")
         oprime = None # Shouldn't be used anywhere
         return (o, oprime)
 
     def not_condition(self, o, c):
         o = self.reuse_if_possible(o, c[0])
-        if not isinstance(c[1], Zero):
-            warning("Differentiating a conditional with a condition "\
-                        "that depends on the differentiation variable."\
-                        "This is probably not a good idea!")
+        #if not (isinstance(c[1], Zero) or c[1] is None):
+        #    warning("Differentiating a conditional with a condition "\
+        #                "that depends on the differentiation variable."\
+        #                "Assuming continuity of conditional. The condition "\
+        #                "will not be differentiated.")
         oprime = None # Shouldn't be used anywhere
         return (o, oprime)
 
     def conditional(self, o, c, t, f):
         o = self.reuse_if_possible(o, c[0], t[0], f[0])
         if isinstance(t[1], Zero) and isinstance(f[1], Zero):
-            fi = o.free_indices()
-            fid = subdict(o.index_dimensions(), fi)
-            op = Zero(o.shape(), fi, fid)
+            tp = t[1] # Assuming t[1] and f[1] have the same indices here, which should be the case
+            fi = tp.free_indices()
+            fid = subdict(tp.index_dimensions(), fi)
+            op = Zero(tp.shape(), fi, fid)
         else:
-            op = conditional(c[0], t[1], f[1])
+            op = conditional(c[0], 1, 0)*t[1] + conditional(c[0], 0, 1)*f[1]
         return (o, op)
 
     # --- Other derivatives
@@ -558,7 +586,10 @@ class ForwardAD(Transformer):
     def derivative(self, o):
         error("This should never occur.")
 
-    def spatial_derivative(self, o):
+    def grad(self, o):
+        error("FIXME")
+
+    def xspatial_derivative(self, o): # FIXME: Translate to grad situation
         # If we hit this type, it has already been propagated
         # to a terminal, so we can simply apply our derivative
         # to its operand since differentiation commutes.
@@ -570,7 +601,7 @@ class ForwardAD(Transformer):
         if fp.is_cellwise_constant():
             sh = fp.shape()
             fi = fp.free_indices()
-            idims = fp.index_dimensions()
+            idims = dict(fp.index_dimensions())
             j, = ii
             if isinstance(j, Index) and j not in idims:
                 fi = fi + (j,)
@@ -581,41 +612,44 @@ class ForwardAD(Transformer):
             oprime = SpatialDerivative(fp, ii)
         return (o, oprime)
 
-class SpatialAD(ForwardAD):
-    def __init__(self, spatial_dim, index, cache=None):
-        index, = index
-        if isinstance(index, Index):
-            vfi = (index,)
-            vid = { index: spatial_dim }
-        else:
-            vfi = ()
-            vid = {}
-        ForwardAD.__init__(self, spatial_dim, var_shape=(), var_free_indices=vfi,
-                           var_index_dimensions=vid, cache=cache)
-        self._index = index
+class GradAD(ForwardAD):
+    def __init__(self, spatial_dim, cache=None):
+        ForwardAD.__init__(self, spatial_dim,
+                           var_shape=(spatial_dim,),
+                           var_free_indices=(),
+                           var_index_dimensions={},
+                           cache=cache)
+        self._Id = Identity(spatial_dim)
 
     def spatial_coordinate(self, o):
-        # Need to define dx_i/dx_j = delta_ij?
-        if o.shape() == ():
-            return (o, IntValue(1))
-        else:
-            I = Identity(self._spatial_dim)
-            oprime = I[:, self._index]
-            return (o, oprime)
+        "Gradient of x w.r.t. x is Id."
+        return (o, self._Id)
 
-    # This is implicit for all terminals, but just to make this clear:
+    # This is implicit for all terminals, but just to make this clear to the reader:
     facet_normal = ForwardAD.terminal # returns zero
     constant = ForwardAD.terminal # returns zero
 
     def form_argument(self, o):
-        # Using this index in here may collide with the
-        # same index on the outside! (There's a check for
-        # this situation in index_sum above.)
-        #oprime = o.dx(self._index) # Not using this syntax because it would create a new IndexSum
+        "Represent grad(f) as Grad(f)."
+        # Collapse gradient of cellwise function to zero
         if o.is_cellwise_constant():
             return self.terminal(o)
-        oprime = SpatialDerivative(o, self._index)
-        return (o, oprime)
+        return (o, Grad(o))
+
+    def grad(self, o):
+        "Represent grad(grad(f)) as Grad(Grad(f))."
+
+        # TODO: Not sure how to detect that gradient of f is cellwise constant.
+        #       Can we trust element degrees?
+        #if o.is_cellwise_constant():
+        #    return self.terminal(o)
+        # TODO: Maybe we can ask "f.has_derivatives_of_order(n)" to check
+        #       if we should make a zero here?
+
+        f, = o.operands()
+        ufl_assert(isinstance(f, (Grad,Terminal)),
+                   "Expecting derivatives of child to be already expanded.")
+        return (o, Grad(o))
 
 class VariableAD(ForwardAD):
     def __init__(self, spatial_dim, var, cache=None):
@@ -625,6 +659,11 @@ class VariableAD(ForwardAD):
                            cache=cache)
         self._variable = var
 
+    def grad(self, o):
+        # If we hit this type, it has already been propagated
+        # to a coefficient, so it cannot depend on the variable. # FIXME: Assert this!
+        return self.terminal(o)
+
     def variable(self, o):
         # Check cache
         e, l = o.operands()
@@ -667,7 +706,7 @@ class CoefficientAD(ForwardAD):
         debug("self._v = %s" % self._v)
 
         # Find o among w
-        for (w, v) in zip(self._w, self._v):
+        for (w, v) in izip(self._w, self._v):
             if o == w:
                 return (w, v)
 
@@ -690,7 +729,7 @@ class CoefficientAD(ForwardAD):
             # Since we may actually have a tuple of oprimes and vs in a
             # 'mixed' space, sum over them all to get the complete inner
             # product. Using indices to define a non-compound inner product.
-            for (oprime, v) in zip(oprimes, self._v):
+            for (oprime, v) in izip(oprimes, self._v):
                 so, oi = as_scalar(oprime)
                 rv = len(v.shape())
                 oi1 = oi[:-rv]
@@ -708,6 +747,148 @@ class CoefficientAD(ForwardAD):
 
         return (o, oprimesum)
 
+    def grad(self, g):
+        # If we hit this type, it has already been propagated
+        # to a coefficient (or grad of a coefficient), # FIXME: Assert this!
+        # so we need to take the gradient of the variation or return zero.
+        # Complications occur when dealing with derivatives w.r.t. single components...
+
+        # Figure out how many gradients are around the inner terminal
+        ngrads = 0
+        o = g
+        while isinstance(o, Grad):
+            o, = o.operands()
+            ngrads += 1
+        if not isinstance(o, FormArgument):
+            error("Expecting gradient of a FormArgument, not %r" % (o,))
+
+        def apply_grads(f):
+            if not isinstance(f, FormArgument):
+                print ','*60
+                print f
+                print o
+                print g
+                print ','*60
+                error("What?")
+            for i in range(ngrads):
+                f = Grad(f)
+            return f
+
+        # Find o among all w without any indexing, which makes this easy
+        for (w, v) in izip(self._w, self._v):
+            if o == w and isinstance(v, FormArgument):
+                # Case: d/dt [w + t v]
+                return (g, apply_grads(v))
+
+        # If o is not among coefficient derivatives, return do/dw=0
+        gprimesum = Zero(g.shape())
+
+        def analyse_variation_argument(v):
+            # Analyse variation argument
+            if isinstance(v, FormArgument):
+                # Case: d/dt [w[...] + t v]
+                vval, vcomp = v, ()
+            elif isinstance(v, Indexed):
+                # Case: d/dt [w + t v[...]]
+                # Case: d/dt [w[...] + t v[...]]
+                vval, vcomp = v.operands()
+                vcomp = tuple(vcomp)
+            else:
+                error("Expecting argument or component of argument.")
+            ufl_assert(all(isinstance(k, FixedIndex) for k in vcomp),
+                       "Expecting only fixed indices in variation.")
+            return vval, vcomp
+
+        def compute_gprimeterm(ngrads, vval, vcomp, wshape, wcomp):
+            # Apply gradients directly to argument vval,
+            # and get the right indexed scalar component(s)
+            kk = indices(ngrads)
+            Dvkk = apply_grads(vval)[vcomp+kk]
+            # Place scalar component(s) Dvkk into the right tensor positions
+            if wshape:
+                Ejj, jj = unit_indexed_tensor(wshape, wcomp)
+            else:
+                Ejj, jj = 1, ()
+            gprimeterm = as_tensor(Ejj*Dvkk, jj+kk)
+            return gprimeterm
+
+        # Accumulate contributions from variations in different components
+        for (w, v) in izip(self._w, self._v):
+
+            # Analyse differentiation variable coefficient
+            if isinstance(w, FormArgument):
+                if not w == o: continue
+                wshape = w.shape()
+
+                if isinstance(v, FormArgument):
+                    # Case: d/dt [w + t v]
+                    return (g, apply_grads(v))
+
+                elif isinstance(v, ListTensor):
+                    # Case: d/dt [w + t <...,v,...>]
+                    for wcomp, vsub in unwrap_list_tensor(v):
+                        if not isinstance(vsub, Zero):
+                            vval, vcomp = analyse_variation_argument(vsub)
+                            gprimesum = gprimesum + compute_gprimeterm(ngrads, vval, vcomp, wshape, wcomp)
+
+                else:
+                    ufl_assert(wshape == (), "Expecting scalar coefficient in this branch.")
+                    # Case: d/dt [w + t v[...]]
+                    wval, wcomp = w, ()
+
+                    vval, vcomp = analyse_variation_argument(v)
+                    gprimesum = gprimesum + compute_gprimeterm(ngrads, vval, vcomp, wshape, wcomp)
+
+            elif isinstance(w, Indexed): # This path is tested in unit tests, but not actually used?
+                # Case: d/dt [w[...] + t v[...]]
+                # Case: d/dt [w[...] + t v]
+                wval, wcomp = w.operands()
+                if not wval == o: continue
+                assert isinstance(wval, FormArgument)
+                ufl_assert(all(isinstance(k, FixedIndex) for k in wcomp),
+                           "Expecting only fixed indices in differentiation variable.")
+                wshape = wval.shape()
+
+                vval, vcomp = analyse_variation_argument(v)
+                gprimesum = gprimesum + compute_gprimeterm(ngrads, vval, vcomp, wshape, wcomp)
+
+            else:
+                error("Expecting coefficient or component of coefficient.")
+
+        # FIXME: Handle other coefficient derivatives: oprimes = self._cd._data.get(o)
+
+        if 0:
+            oprimes = self._cd._data.get(o)
+            if oprimes is None:
+                if self._cd._data:
+                    # TODO: Make it possible to silence this message in particular?
+                    #       It may be good to have for debugging...
+                    warning("Assuming d{%s}/d{%s} = 0." % (o, self._w))
+            else:
+                # Make sure we have a tuple to match the self._v tuple
+                if not isinstance(oprimes, tuple):
+                    oprimes = (oprimes,)
+                    ufl_assert(len(oprimes) == len(self._v), "Got a tuple of arguments, "+\
+                                   "expecting a matching tuple of coefficient derivatives.")
+
+                # Compute dg/dw_j = dg/dw_h : v.
+                # Since we may actually have a tuple of oprimes and vs in a
+                # 'mixed' space, sum over them all to get the complete inner
+                # product. Using indices to define a non-compound inner product.
+                for (oprime, v) in izip(oprimes, self._v):
+                    error("FIXME: Figure out how to do this with ngrads")
+                    so, oi = as_scalar(oprime)
+                    rv = len(v.shape())
+                    oi1 = oi[:-rv]
+                    oi2 = oi[-rv:]
+                    prod = so*v[oi2]
+                    if oi1:
+                        gprimesum += as_tensor(prod, oi1)
+                    else:
+                        gprimesum += prod
+
+        return (g, gprimesum)
+
     def variable(self, o):
         # Check variable cache to reuse previously transformed variable if possible
         e, l = o.operands()
@@ -727,9 +908,9 @@ class CoefficientAD(ForwardAD):
         self._variable_cache[l] = c
         return c
 
-def compute_spatial_forward_ad(expr, dim):
-    f, v = expr.operands()
-    alg = SpatialAD(dim, v)
+def compute_grad_forward_ad(expr, dim):
+    f, = expr.operands()
+    alg = GradAD(dim)
     e, ediff = alg.visit(f)
     return ediff
 
@@ -745,20 +926,40 @@ def compute_coefficient_forward_ad(expr, dim):
     e, ediff = alg.visit(f)
     return ediff
 
-def forward_ad(expr, dim):
-    """Assuming expr is a derivative and contains no other
-    unresolved derivatives, apply forward mode AD and
-    return the computed derivative."""
-    if isinstance(expr, SpatialDerivative):
-        result = compute_spatial_forward_ad(expr, dim)
-    elif isinstance(expr, VariableDerivative):
-        result = compute_variable_forward_ad(expr, dim)
-    elif isinstance(expr, CoefficientDerivative):
-        result = compute_coefficient_forward_ad(expr, dim)
+def apply_nested_forward_ad(expr, dim):
+    if isinstance(expr, Terminal):
+        # A terminal needs no differentiation applied
+        return expr
     else:
-        error("This shouldn't happen: expr is %s" % repr(expr))
-    return result
+        # Store the original expression here, because
+        # reconstruct below may simplify and make expr
+        # get the type of a child, which may be a derivative,
+        # even if preexpr here is not a derivative.
+        preexpr = expr
 
+        # Apply AD recursively to children
+        preops = expr.operands()
+        postops = tuple(apply_nested_forward_ad(o, dim) for o in preops)
+
+        need_reconstruct = not (preops == postops) # FIXME: Is this efficient? O(n)?
+        if need_reconstruct:
+            expr = expr.reconstruct(*postops)
+
+        # Preliminary result
+        result = expr
+
+        # Check if this node is a derivative, if so apply AD to it
+        if isinstance(preexpr, Derivative):
+            if isinstance(expr, Grad):
+                result = compute_grad_forward_ad(expr, dim)
+            elif isinstance(preexpr, VariableDerivative):
+                result = compute_variable_forward_ad(expr, dim)
+            elif isinstance(preexpr, CoefficientDerivative):
+                result = compute_coefficient_forward_ad(expr, dim)
+            else:
+                error("This shouldn't happen: expr is %s" % repr(expr))
+        # Return the expanded result
+        return result
 
 # TODO: We could expand only the compound objects that have no rule
 #       before differentiating, to allow the AD to work on a coarser graph
@@ -772,43 +973,50 @@ class UnusedADRules(object):
         oprime = o.reconstruct(fp, v)
         return (o, oprime)
 
-    # --- Compound operators
-
-    def commute(self, o, a):
-        "This should work for all single argument operators that commute with d/dw."
-        aprime = a[1]
-        return (o, o.reconstruct(aprime))
-
-    transposed = commute
-    trace = commute
-    deviatoric = commute
-
-    # TODO: nabla_div, nabla_grad
-    div  = commute
-    curl = commute
-    def grad(self, o, a):
-        a, aprime = a
-        c = aprime.cell()
-        if c is None or c.is_undefined():
-            oprime = self._make_zero_diff(o)
-        else:
-            oprime = o.reconstruct(aprime)
-        return (o, oprime)
+    # --- Tensor algebra (compound types)
 
     def outer(self, o, a, b):
         a, ap = a
         b, bp = b
-        return (o, outer(ap, b) + outer(a, bp))
+        return (o, outer(ap, b) + outer(a, bp)) # FIXME: Not valid for derivatives w.r.t. nonscalar variables!
 
     def inner(self, o, a, b):
         a, ap = a
         b, bp = b
-        return (o, inner(ap, b) + inner(a, bp))
+        # NB! Using b : ap because derivative axis should be
+        # last, in case of nonscalar differentiation variable!
+        return (o, inner(b, ap) + inner(a, bp)) # FIXME: Not correct, inner requires equal shapes!
 
     def dot(self, o, a, b):
         a, ap = a
         b, bp = b
-        return (o, dot(ap, b) + dot(a, bp))
+        # NB! Using b . ap because derivative axis should be
+        # last, in case of nonscalar differentiation variable!
+        return (o, dot(b, ap) + dot(a, bp))
+
+    def commute(self, o, a):
+        "This should work for all single argument operators that commute with d/dw with w scalar."
+        aprime = a[1]
+        return (o, o.reconstruct(aprime))
+
+    # FIXME: Not true for derivatives w.r.t. nonscalar variables...
+    transposed = commute
+    trace = commute
+    deviatoric = commute
+
+    # --- Compound differential operators, probably do not want...
+
+    # FIXME: nabla_div, nabla_grad
+    div  = commute
+    curl = commute
+    def grad(self, o, a):
+        a, aprime = a
+        cell = aprime.cell()
+        if cell is None: # FIXME
+            oprime = self._make_zero_diff(o)
+        else:
+            oprime = o.reconstruct(aprime)
+        return (o, oprime)
 
 class UnimplementedADRules(object):
 
