@@ -1,7 +1,7 @@
 """A collection of utility algorithms for printing
 of UFL objects, mostly intended for debugging purposes."""
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -52,10 +52,11 @@ def form_info(form):
     bf = extract_arguments(form)
     cf = extract_coefficients(form)
 
-    ci = form.cell_integrals()
-    ei = form.exterior_facet_integrals()
-    ii = form.interior_facet_integrals()
-    mi = form.macro_cell_integrals()
+    ci = form.integrals(Measure.CELL)
+    ei = form.integrals(Measure.EXTERIOR_FACET)
+    ii = form.integrals(Measure.INTERIOR_FACET)
+    pi = form.integrals(Measure.POINT)
+    mi = form.integrals(Measure.MACRO_CELL)
 
     s  = "Form info:\n"
     s += "  rank:                          %d\n" % len(bf)
@@ -63,6 +64,7 @@ def form_info(form):
     s += "  num_cell_integrals:            %d\n" % len(ci)
     s += "  num_exterior_facet_integrals:  %d\n" % len(ei)
     s += "  num_interior_facet_integrals:  %d\n" % len(ii)
+    s += "  num_point_integrals:           %d\n" % len(pi)
     s += "  num_macro_cell_integrals:      %d\n" % len(mi)
 
     for f in cf:
@@ -106,13 +108,14 @@ def tree_format(expression, indentation=0, parentheses=True):
     s = ""
 
     if isinstance(expression, Form):
-        ci = expression.cell_integrals()
-        ei = expression.exterior_facet_integrals()
-        ii = expression.interior_facet_integrals()
-        mi = expression.macro_cell_integrals()
+        ci = expression.integrals(Measure.CELL)
+        ei = expression.integrals(Measure.EXTERIOR_FACET)
+        ii = expression.integrals(Measure.INTERIOR_FACET)
+        pi = expression.integrals(Measure.POINT)
+        mi = expression.integrals(Measure.MACRO_CELL)
         ind = _indent_string(indentation)
         s += ind + "Form:\n"
-        s += "\n".join(tree_format(itg, indentation+1, parentheses) for itg in chain(ci, ei, ii, mi))
+        s += "\n".join(tree_format(itg, indentation+1, parentheses) for itg in chain(ci, ei, ii, pi, mi))
 
     elif isinstance(expression, Integral):
         ind = _indent_string(indentation)
