@@ -2,7 +2,7 @@
 either converting UFL expressions to new UFL expressions or
 converting UFL expressions to other representations."""
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -25,7 +25,6 @@ converting UFL expressions to other representations."""
 # First added:  2008-05-07
 # Last changed: 2011-10-21
 
-import os
 from itertools import chain
 
 from ufl.log import error, warning
@@ -35,18 +34,20 @@ from ufl.permutation import compute_indices
 
 # All classes:
 from ufl.variable import Variable
-from ufl.indexing import Indexed, Index, FixedIndex
+from ufl.indexing import Index, FixedIndex
+from ufl.indexed import Indexed
 from ufl.tensors import ListTensor, ComponentTensor
 from ufl.algebra import Sum, Product, Division, Power, Abs
 from ufl.indexsum import IndexSum
 from ufl.tensoralgebra import Transposed, Outer, Inner, Dot, Cross, Trace, Determinant, Inverse, Deviatoric, Cofactor
-from ufl.mathfunctions import Sqrt, Exp, Ln, Cos, Sin, Tan, Acos, Asin, Atan, Erf, BesselJ, BesselY, BesselI, BesselK
+from ufl.mathfunctions import Sqrt, Exp, Ln, Cos, Sin, Tan, Cosh, Sinh, Tanh, Acos, Asin, Atan, Erf, BesselJ, BesselY, BesselI, BesselK
 from ufl.restriction import PositiveRestricted, NegativeRestricted
-from ufl.differentiation import SpatialDerivative, VariableDerivative, Grad, Div, Curl, NablaGrad, NablaDiv
+from ufl.differentiation import VariableDerivative, Grad, Div, Curl, NablaGrad, NablaDiv
 from ufl.conditional import EQ, NE, LE, GE, LT, GT, Conditional
 from ufl.form import Form
 from ufl.integral import Measure
 from ufl.classes import terminal_classes
+from ufl.domains import Domain, Region
 
 # Other algorithms:
 from ufl.algorithms.preprocess import preprocess
@@ -54,7 +55,7 @@ from ufl.algorithms.analysis import extract_variables
 from ufl.algorithms.formfiles import load_forms
 from ufl.algorithms.latextools import align, document, verbatim
 
-from ufl.algorithms.transformations import expand_compounds, mark_duplications, Transformer
+from ufl.algorithms.transformer import Transformer
 from ufl.algorithms.graph import build_graph, partition, extract_outgoing_vertex_connections
 
 
@@ -76,11 +77,11 @@ def build_precedence_map():
     precedence_list.append((Conditional,))
     precedence_list.append((LE, GT, GE, NE, EQ, LT))
 
-    precedence_list.append((Div, Grad, NablaGrad, NablaDiv, Curl, SpatialDerivative, VariableDerivative,
+    precedence_list.append((Div, Grad, NablaGrad, NablaDiv, Curl, VariableDerivative,
                             Determinant, Trace, Cofactor, Inverse, Deviatoric))
     precedence_list.append((Product, Division, Cross, Dot, Outer, Inner))
     precedence_list.append((Indexed, Transposed, Power))
-    precedence_list.append((Abs, Cos, Exp, Ln, Sin, Sqrt, Tan, Acos, Asin, Atan, Erf, BesselJ, BesselY, BesselI, BesselK))
+    precedence_list.append((Abs, Cos, Cosh, Exp, Ln, Sin, Sinh, Sqrt, Tan, Tanh, Acos, Asin, Atan, Erf, BesselJ, BesselY, BesselI, BesselK))
     precedence_list.append((Variable,))
     precedence_list.append(terminal_classes)
 
@@ -187,12 +188,6 @@ class Expression2LatexHandler(Transformer):
     def indexed(self, o, a, b):
         return "{%s}_{%s}" % (par(a), b)
 
-    def spatial_derivative(self, o, f, ii):
-        ii = o.operands()[1]
-        nom = r"\partial%s" % par(f)
-        denom = format_multi_index(ii, formatstring="\partial{}x_{%s}")
-        return r"\frac{%s}{%s}" % (nom, denom)
-
     def variable_derivative(self, o, f, v):
         nom   = r"\partial%s" % par(f)
         denom = r"\partial%s" % par(v)
@@ -236,6 +231,15 @@ class Expression2LatexHandler(Transformer):
     def tan(self, o, f):
         return r"\tan{%s}" % par(f)
 
+    def cosh(self, o, f):
+        return r"\cosh{%s}" % par(f)
+
+    def sinh(self, o, f):
+        return r"\sinh{%s}" % par(f)
+
+    def tanh(self, o, f):
+        return r"\tanh{%s}" % par(f)
+
     def acos(self, o, f):
         return r"\arccos{%s}" % par(f)
 
@@ -371,13 +375,15 @@ def element2latex(element):
 domain_strings = { Measure.CELL: r"\Omega",
                    Measure.EXTERIOR_FACET: r"\Gamma^{ext}",
                    Measure.INTERIOR_FACET: r"\Gamma^{int}",
+                   Measure.POINT: r"\Gamma^{point}",
                    Measure.MACRO_CELL: r"\Omega^{macro}",
                    Measure.SURFACE: r"\Gamma^{surface}",
                  }
-default_domain_string = "D"
+default_domain_string = "d(?)"
 dx_strings = { Measure.CELL: "dx",
                Measure.EXTERIOR_FACET: "ds",
                Measure.INTERIOR_FACET: "dS",
+               Measure.POINT: "dP",
                Measure.MACRO_CELL: "dE",
                Measure.SURFACE: "dc",
              }
@@ -422,11 +428,12 @@ def form2latex(form, formdata):
 
     # Define variables
     handled_variables = set()
-    integrals = list(chain(form.cell_integrals(),
-                           form.exterior_facet_integrals(),
-                           form.interior_facet_integrals(),
-                           form.macro_cell_integrals(),
-                           form.surface_integrals()))
+    integrals = list(chain(form.integrals(Measure.CELL),
+                           form.integrals(Measure.EXTERIOR_FACET),
+                           form.integrals(Measure.INTERIOR_FACET),
+                           form.integrals(Measure.POINT),
+                           form.integrals(Measure.MACRO_CELL),
+                           form.integrals(Measure.SURFACE)))
     ufl_assert(len(integrals) == len(form.integrals()),
                "Not handling non-standard integral types!")
     lines = []
@@ -449,18 +456,33 @@ def form2latex(form, formdata):
 
     # Define form as sum of integrals
     lines = []
-    a = signature; p = ""
+    a = signature
+    p = ""
     for itg in integrals:
         # TODO: Get list of expression strings instead of single expression!
         integrand_string = expression2latex(itg.integrand(), formdata.argument_names, formdata.coefficient_names)
-        dtyp = itg.measure().domain_type()
-        dstr = domain_strings.get(dtyp, default_domain_string)
-        b = p + "\\int_{%s_%d}" % (dstr, itg.measure().domain_id())
-        dxstr = Measure._domain_types[dtyp]
-        dxstr = dx_strings[dtyp]
+
+        domain_type = itg.domain_type()
+        dstr = domain_strings[domain_type]
+
+        domain_id = itg.domain_id()
+        if isinstance(domain_id, int):
+            dstr += "_{%d}" % domain_id
+        elif isinstance(domain_id, Domain) or domain_id == Measure.DOMAIN_ID_EVERYWHERE:
+            pass
+        elif domain_id == Measure.DOMAIN_ID_OTHERWISE:
+            dstr += "_{\text{oth}}"
+        elif isinstance(domain_id, tuple):
+            dstr += "_{%s}" % domain_id
+
+        b = p + "\\int_{%s}" % (dstr,)
+        dxstr = Measure._domain_types[domain_type]
+        dxstr = dx_strings[domain_type]
         c = "{ %s } \\,%s" % (integrand_string, dxstr)
         lines.append((a, b, c))
-        a = "{}"; p = "{}+ "
+        a = "{}"
+        p = "{}+ "
+
     sections.append(("Form", align(lines)))
 
     return sections
@@ -609,9 +631,8 @@ def form2code2latex(formdata):
     sections = [(title, body)]
 
     # Render each integral as a separate section
-    for itg in formdata.form.cell_integrals():
-        m = itg.measure()
-        title = "%s integral over domain %d" % (m.domain_type(), m.domain_id())
+    for itg in formdata.form.integrals(Measure.CELL):
+        title = "%s integral over domain %d" % (itg.domain_type(), itg.domain_id())
 
         G, partitions = integrand2code(itg.integrand(), formdata)
 
