@@ -1,7 +1,7 @@
 """This module defines the Coefficient class and a number
 of related classes, including Constant."""
 
-# Copyright (C) 2008-2011 Martin Sandve Alnes
+# Copyright (C) 2008-2013 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -25,24 +25,21 @@ of related classes, including Constant."""
 
 from ufl.log import warning
 from ufl.assertions import ufl_assert
-from ufl.common import Counted, product
 from ufl.terminal import FormArgument
 from ufl.finiteelement import FiniteElementBase, FiniteElement, VectorElement, TensorElement
 from ufl.split_functions import split
-from ufl.geometry import as_cell
 
 # --- The Coefficient class represents a coefficient in a form ---
 
-class Coefficient(FormArgument, Counted):
+class Coefficient(FormArgument):
     """UFL form argument type: Representation of a form coefficient."""
 
     # Slots are disabled here because they cause trouble in PyDOLFIN multiple inheritance pattern:
-    #__slots__ = ("_element", "_repr", "_gradient", "_derivatives")
+    #__slots__ = ("_count", "_element", "_repr", "_gradient", "_derivatives")
     _globalcount = 0
 
     def __init__(self, element, count=None):
-        FormArgument.__init__(self)
-        Counted.__init__(self, count, Coefficient)
+        FormArgument.__init__(self, count, Coefficient)
         ufl_assert(isinstance(element, FiniteElementBase),
             "Expecting a FiniteElementBase instance.")
         self._element = element
@@ -81,6 +78,9 @@ class Coefficient(FormArgument, Counted):
     def cell(self):
         return self._element.cell()
 
+    def domain(self):
+        return self._element.domain()
+
     def __str__(self):
         count = str(self._count)
         if len(count) == 1:
@@ -94,7 +94,12 @@ class Coefficient(FormArgument, Counted):
         return self._repr
 
     def __eq__(self, other):
-        return isinstance(other, Coefficient) and self._element == other._element and self._count == other._count
+        if not isinstance(other, Coefficient):
+            return False
+        if self is other:
+            return True
+        return (self._count == other._count and
+                self._element == other._element)
 
 # --- Subclasses for defining constant coefficients without specifying element ---
 
@@ -107,13 +112,13 @@ class Constant(ConstantBase):
     """UFL value: Represents a globally constant scalar valued coefficient."""
     __slots__ = ()
 
-    def __init__(self, cell, count=None):
-        e = FiniteElement("Real", cell, 0)
+    def __init__(self, domain, count=None):
+        e = FiniteElement("Real", domain, 0)
         ConstantBase.__init__(self, e, count)
-        self._repr = "Constant(%r, %r)" % (e.cell(), self._count)
+        self._repr = "Constant(%r, %r)" % (e.domain(), self._count)
 
     def _reconstruct(self, element, count):
-        return Constant(element.cell(), count)
+        return Constant(element.domain(), count)
 
     def __str__(self):
         count = str(self._count)
@@ -126,14 +131,14 @@ class VectorConstant(ConstantBase):
     """UFL value: Represents a globally constant vector valued coefficient."""
     __slots__ = ()
 
-    def __init__(self, cell, dim=None, count=None):
-        e = VectorElement("Real", cell, 0, dim)
+    def __init__(self, domain, dim=None, count=None):
+        e = VectorElement("Real", domain, 0, dim)
         ConstantBase.__init__(self, e, count)
         ufl_assert(self._repr is None, "Repr should not have been set yet!")
-        self._repr = "VectorConstant(%r, %r, %r)" % (e.cell(), e.value_shape()[0], self._count)
+        self._repr = "VectorConstant(%r, %r, %r)" % (e.domain(), e.value_shape()[0], self._count)
 
     def _reconstruct(self, element, count):
-        return VectorConstant(element.cell(), element.value_shape()[0], count)
+        return VectorConstant(element.domain(), element.value_shape()[0], count)
 
     def __str__(self):
         count = str(self._count)
@@ -146,15 +151,15 @@ class TensorConstant(ConstantBase):
     """UFL value: Represents a globally constant tensor valued coefficient."""
     __slots__ = ()
 
-    def __init__(self, cell, shape=None, symmetry=None, count=None):
-        e = TensorElement("Real", cell, 0, shape=shape, symmetry=symmetry)
+    def __init__(self, domain, shape=None, symmetry=None, count=None):
+        e = TensorElement("Real", domain, 0, shape=shape, symmetry=symmetry)
         ConstantBase.__init__(self, e, count)
         ufl_assert(self._repr is None, "Repr should not have been set yet!")
-        self._repr = "TensorConstant(%r, %r, %r, %r)" % (e.cell(), e.value_shape(), e._symmetry, self._count)
+        self._repr = "TensorConstant(%r, %r, %r, %r)" % (e.domain(), e.value_shape(), e._symmetry, self._count)
 
     def _reconstruct(self, element, count):
         e = element
-        return TensorConstant(e.cell(), e.value_shape(), e._symmetry, count)
+        return TensorConstant(e.domain(), e.value_shape(), e._symmetry, count)
 
     def __str__(self):
         count = str(self._count)
